/* (C) Copyright 2012 Greensoft Pty Ltd, Australia (ACN 148 917 999). All rights reserved. */
package com.greensoft.objectdb.test.common;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Provides common logging.
 *
 * The logging uses efficient static lookup of shared loggers for all classes in the system.
 *
 * There is simple debug status management.
 *
 * @author darrenkelly
 */
public class All_ implements Serializable {

    private static final Map<Class,Logger> loggers = new HashMap<Class,Logger>();

    static private Logger getLoggerFor(Class clazz) {
        //if (clazz == null) //TODO warn;
        if (loggers.containsKey(clazz)) return loggers.get(clazz);
        Logger l = Logger.getLogger(clazz.getName());
        loggers.put(clazz,l);
        return l;
    }

    /** Reimplement in those subclasses that can provide an object identifier string */
    protected String $id() {return "";}
    
    private String $simple;
    
    private String $simple() {
        if ($simple==null) $simple = this.getClass().getSimpleName();
        return $simple;
    }

    protected void log_error(String s) {
        myLogger().log(Level.SEVERE, s);
    }

    protected void log_error(String $method, String s) {
        myLogger().log(Level.SEVERE, "{0} [{1}]: {2}: {3}", new Object[]{$simple(),$id(), $method, s});
    }

    protected void log_error(Exception e) {
        myLogger().log(Level.SEVERE, e.getMessage());
        e.printStackTrace(); //TODO ?remove Throwable.printStackTrace();
    }

    protected void log_info(String s) {
        myLogger().log(Level.INFO, "{0} [{1}]: {2}", new Object[]{$simple(), $id(), s});
    }

    protected void log_info(String $method, String s) {
        myLogger().log(Level.INFO, "{0} [{1}]: {2}: {3}", new Object[]{$simple(),$id(),$method, s});
    }

    protected void log_info(String $method, String name, Object value) {
        myLogger().log(Level.INFO, "{0} [{1}]: {2}: {3}({4})", new Object[]{$simple(),$id(),$method, name, value});
    }

    protected void log_warn(String $method, String s) {
        myLogger().log(Level.WARNING, "{0} [{1}]: {2}: {3}", new Object[]{$simple(), $id(), $method, s});
    }

    protected void log_warn(String s) {
        myLogger().log(Level.WARNING, "{0} [{1}]: {2}", new Object[]{$simple(), $id(), s});
    }

    private boolean debug = false;
    public boolean isDebug() {
        return debug;
    }
    public void setDebug(boolean debug) {
        this.debug = debug;
    }

    protected void log_debug(String $method, String name, Object value) {
        if (isDebug()) myLogger().log(Level.INFO, "DEBUG: {0} [{1}]: {2}: {3}({4})", new Object[]{$simple(),$id(),$method, name, value});
    }

    protected void log_debug(String $method, String s) {
        if (isDebug()) myLogger().log(Level.INFO, "DEBUG: {0} [{1}]: {2}: {3}", new Object[]{$simple(), $id(), $method, s});
    }

    protected void log_debug(String s) {
        if (isDebug()) myLogger().log(Level.INFO, "DEBUG: {0} [{1}]: {2}", new Object[]{$simple(), $id(), s});
    }

    protected Logger myLogger() {
        return getLoggerFor(this.getClass());
    }

}
