/* (C) Copyright 2012 Greensoft Pty Ltd, Australia (ACN 148 917 999). All rights reserved. */
package com.greensoft.objectdb.test.console;

import com.greensoft.objectdb.test.entity.*;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

/**
 *
 * @author darrenkelly
 */
public class Main
{
    private Format format;

    protected Format format() {
        if (format == null) {
            format = new Format();
        }
        return format;
    }
    
    final static private boolean DO_TRY_FORCE_NOSHOW_LAZY = true;

    protected void createProject(EntityManagerFactory emf) {
        
        format().outHeader2("Create Project and Element model ...");
        Project project = new Project(null, "project"); //NB: Project has no project and no owner

        Element model = new Element(project, project, "model");//NB: model has Project as owner        
        project.addOwnedElement(model);

        Element example = new Element(model, project, "example element");
        example.setHelp("Note owner of this is a model Element");
        model.addOwnedElement(example);

        /*
        Element group = new Element(model, project, "group");
        model.addOwnedElement(group);
        Element child1 = new Element(group, project, "child 1");
        group.addOwnedElement(child1);
        Element child2 = new Element(group, project, "child 2");
        group.addOwnedElement(child2);

        Constraint constraint1 = new Constraint(model, project, "constraint 1");
        model.addOwnedElement(constraint1);
        Element constrainedElement1 = new Element(model, project, "constrained element 1");
        model.addOwnedElement(constrainedElement1);
        constrainedElement1.setBody("Body is a LAZY @LOB");
        constrainedElement1.addConstraint(constraint1);
        constraint1.addConstrainedElement(constrainedElement1);

        TagSet tagSet = new TagSet(model, project, "tag set");
        model.addOwnedElement(tagSet);
        Tag tag1 = new Tag(tagSet, project, "tag");
        //tagSet.addOwnedElement(tag1);
        tagSet.addOwnedTag(tag1);
        constrainedElement1.addTag(tag1);
        tag1.addTaggedElement(constrainedElement1);

        Element childOfConstrainedElement1 = new Element(constrainedElement1, project, "child of constrained element 1");
        constrainedElement1.addOwnedElement(childOfConstrainedElement1);

        Source source = new Source(model, project, "source");
        model.addOwnedElement(source);
        constrainedElement1.setSource(source);

        Element unmapped = new Element(model, project, "unmapped");
        constrainedElement1.addUnmapped(unmapped);

        constrainedElement1.setHelp("This Element has 1 Constraint, 1 Tag, 1 owned Element (child), a Source, and 1 unmapped Element");
        */
        
        EntityManager em = emf.createEntityManager();

        format().out("Persist project..");

        em.getTransaction().begin();

        em.persist(project); //rely on Cascade all on ownedElements.

        em.getTransaction().commit();

        format().out("... persist project done.");
        
        format().out("Project has id:" + project.getId());

        em.close();
    }
    
    /** If true the various element views in search and view methods 
     * will be executed after an entity manager is closed,
     * otherwise the view is executed before it is closed.
     */
    final static protected boolean DO_VIEW_AFTER_CLOSE = false;


    protected List<Element> search_and_view_all_by_query_list(EntityManagerFactory emf) {
        format().outHeader2("Search and view by query list");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Element> q = em.createQuery("SELECT e from Element e", Element.class);
        List<Element> elements = q.getResultList();
        if (!DO_VIEW_AFTER_CLOSE) format().viewElements(elements);
        em.close();
        if (DO_VIEW_AFTER_CLOSE) format().viewElements(elements);
        return elements;
    }

    /**
     * Helps test whether it makes a difference to population of lazy fields if
     * entities are loaded using EntityManager.find (as compared with a query result).
     *
     * @param emf EntityManagerFactory
     * @param nElements The number of elements to loop over and then find by id.
     * @return
     */
    protected List<Element> search_and_view_all_find_by_id(EntityManagerFactory emf, int nElements) {
        List<Element> found_by_id = new ArrayList<Element>();

        format().outHeader2("Search and view all find by id");
        EntityManager em = emf.createEntityManager();
        for (long id = 1;//!
                id < nElements - 1; id++) {
            Element e = em.find(Element.class, (Long) id);
            if (e == null) {
                format().warn("Did not find Element for id(" + id + ")");
            } else {
                found_by_id.add(e);
                if (!DO_VIEW_AFTER_CLOSE) {
                    format().out("View before close ..");
                    format().view(e);
                }
            }
        }
        em.close();

        if (DO_VIEW_AFTER_CLOSE) {
            format().out("View after close ..");
            for (Element e : found_by_id) {
                format().view(e);
            }
        }

        return found_by_id;
    }

    protected Element find_by_id_and_view(EntityManagerFactory emf, Long id) {
        format().outHeader2("Find by id(" + id + ") and view");
        EntityManager em = emf.createEntityManager();
        Element e = em.find(Element.class, id);
        //if (DO_TRY_FORCE_NOSHOW_LAZY) em.detach(e);

        if (!DO_VIEW_AFTER_CLOSE) {
            if (e == null) {
                format().warn("Could not find Element with id(" + id + ")");
            } else {
                format().out("Viewing found element before closing EntityManager");
                format().view(e);//should support access to lazy fields
            }
        }

        em.close(); //!

        if (DO_VIEW_AFTER_CLOSE) {
            if (e == null) {
                format().warn("Could not find Element with id(" + id + ")");
            } else {
                format().out("Viewing found element after closed EntityManager");
                format().view(e); //Entitymanager closed, so should NOT support access to lazy fields (but does)
            }
        }

        return e;
    }
    
    private static final boolean DO_CREATE = true;
    
    private static final boolean DO_READ = true;
    
    private static final Long idTest = 1l;

    /**
     * Encapsulate run here so have access to logging
     */
    public void run() {

        format().out("Create EntityManagerFactory ..");
        EntityManagerFactory emf = Persistence.createEntityManagerFactory(                
                //"$objectdb/db/GreensoftObjectdbTestNoweb.odb;user=admin;password=admin"//OK, but no drop
                "objectdb:db/GreensoftObjectdbTestNoweb.tmp;user=admin;password=admin;drop" //OK and drops
                );

        if (DO_CREATE) {
            createProject(emf);
        }

        //List<Element> elements = search_and_view_all_by_query_list(emf);        
        //search_and_view_all_find_by_id(emf,elements.size());
        //search_and_view_all_find_by_id(emf,10);

        if (DO_READ) {
            find_by_id_and_view(emf, idTest);
        }

        emf.close();

        //Experiment: use separate EntityManagerFactory: makes no difference to exposed lazy fields
        /*
         EntityManagerFactory emf2 = Persistence.createEntityManagerFactory(
         "objectdb:db/greensoftObjectdbNowebTest.tmp;user=admin;password=admin;"
         ); 
         find_by_id_and_view(emf2,idTest);
         emf2.close();
         */
    }

    public static void main(String[] args) {
        Main m = new Main();
        m.run();
    }
}
