/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.greensoft.objectdb.test.console;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.*;

/**
 *
 * @author darrenkelly
 */
public class TestUpdateBuilding {

    final static public boolean USE_NODETACH = true;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        if (USE_NODETACH) {
            System.setProperty("objectdb.temp.no-detach", "true");
        }

        String $connection = "objectdb:db/TestUpdateBuilding.tmp;drop";
        EntityManagerFactory emf = Persistence.createEntityManagerFactory($connection);

        Building building = new Building(null, "building");
        EntityManager em1 = emf.createEntityManager();
        em1.getTransaction().begin();
        em1.persist(building);
        em1.getTransaction().commit();
        em1.close();
        Long id = building.id;

        EntityManager em2 = emf.createEntityManager();
        em2.getTransaction().begin();
        em2.find(Building.class, id);
        em2.getTransaction().commit();
        em2.close();

        System.out.println("Building:" + building);
        System.out.println("Building.Garden_Roof:" + building.getP_Garden_Roof());
        System.out.println("Building.Garden_Roof.Substrate_Garden_Roof:" + building.getP_Garden_Roof().getP_Substrate_Garden_Roof());

        //building.name = "changed  name";
        building.getFloatValue().setValue(1f);

        EntityManager em3 = emf.createEntityManager();
        em3.getTransaction().begin();
        em3.merge(building);
        em3.getTransaction().commit();
        em3.close();

        emf.close();
    }

    @Entity
    abstract static public class Element {

        @Id
        @GeneratedValue
        Long id;
        String name;

        protected Element() {
        }

        protected Element(Element owner, String name) {
            this.name = name;
            this.owner = owner;
        }

        @Override
        public String toString() {
            return "{" + getClass().getSimpleName() + "}" + "[" + id + "]\"" + name + "\"";
        }
        private Element owner;

        @ManyToOne(fetch = FetchType.LAZY)
        public Element getOwner() {
            return owner;
        }

        public void setOwner(Element owner) {
            this.owner = owner;
        }
        private List<Element> ownedElements = new ArrayList<Element>();

        @OneToMany(mappedBy = "owner",
        cascade = CascadeType.ALL)
        public List<Element> getOwnedElements() {
            return ownedElements;
        }

        public void setOwnedElements(List<Element> ownedElements) {
            this.ownedElements = ownedElements;
        }

        /**
         * Adds the passed Element iff it is non-null.
         *
         *
         * @param element
         * @return
         */
        public boolean addOwnedElement(Element element) {
            if (element == null) {
                return false;
            }
            if (!getOwnedElements().contains(element)) {
                return getOwnedElements().add(element);
            } else {
                return false;
            }
        }
    }

    @Entity
    abstract static public class DeepValue extends Element {

        protected DeepValue() {
        }

        protected DeepValue(Element owner, String name) {
            super(owner, name);
        }
    }

    @Entity
    static public class FloatValue extends DeepValue {

        public FloatValue() {
        }

        public FloatValue(Element owner, String name) {
            super(owner, name);
        }
        private Float value;

        public Float getValue() {
            return value;
        }

        public void setValue(Float value) {
            this.value = value;
        }
    }

    @Entity
    static public class BooleanValue extends DeepValue {

        public BooleanValue() {
        }

        public BooleanValue(Element owner, String name) {
            super(owner, name);
        }
        private Boolean value;

        public Boolean getValue() {
            return value;
        }

        public void setValue(Boolean value) {
            this.value = value;
        }
    }

    @Entity
    abstract static public class Block extends Element {

        protected Block() {
            init();
        }

        protected Block(Element owner, String name) {
            super(owner, name);
            init();
        }

        private void init() {
            present = new BooleanValue(this, "present");
            this.addOwnedElement(present);
        }
        
        private BooleanValue present;
        @OneToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
        public BooleanValue getPresent() {
            return present;
        }
        public void setPresent(BooleanValue present) {
            this.present = present;
        }
    }

    @Entity
    static public class Building extends Block {

        public Building() {
            init();
        }

        public Building(Element owner, String name) {
            super(owner, name);
            init();
        }

        private void init() {
            
            floatValue = new FloatValue(this,"float value");
            addOwnedElement(floatValue);
            
            p_Garden_Roof = new Garden_Roof(this, "roof garden");
            addOwnedElement(p_Garden_Roof);
        }
        
        private FloatValue floatValue;
        @OneToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
        public FloatValue getFloatValue() {
            return floatValue;
        }
        public void setFloatValue(FloatValue floatValue) {
            this.floatValue = floatValue;
        }
        
                
        private Garden_Roof p_Garden_Roof;
        @OneToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
        public Garden_Roof getP_Garden_Roof() {
            return p_Garden_Roof;
        }
        public void setP_Garden_Roof(Garden_Roof p_Garden_Roof) {
            this.p_Garden_Roof = p_Garden_Roof;
        }
    }

    @Entity
    static public class Garden_Roof extends Block {

        public Garden_Roof() {
            init();
        }

        public Garden_Roof(Element owner, String name) {
            super(owner, name);
            init();
        }

        private void init() {
            p_Substrate_Garden_Roof = new Substrate_Garden_Roof(this, "garden roof substrate");
            addOwnedElement(p_Substrate_Garden_Roof);
        }
        
        private Substrate_Garden_Roof p_Substrate_Garden_Roof;
        @OneToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
        public Substrate_Garden_Roof getP_Substrate_Garden_Roof() {
            return p_Substrate_Garden_Roof;
        }

        public void setP_Substrate_Garden_Roof(Substrate_Garden_Roof p_Substrate_Garden_Roof) {
            this.p_Substrate_Garden_Roof = p_Substrate_Garden_Roof;
        }
    }

    @Entity
    static public class Substrate_Garden_Roof extends Block {

        public Substrate_Garden_Roof() {
            init();
        }

        public Substrate_Garden_Roof(Element owner, String name) {
            super(owner, name);
            init();
        }

        private void init() {
        }
    }
}
