/* (C) Copyright 2012 Greensoft Pty Ltd, Australia (ACN 148 917 999). All rights reserved. */
package com.greensoft.objectdb.test.entity;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;

/**
 *
 * @author darrenkelly
 */
@Entity
public class TagSet extends Element {

    private static final long serialVersionUID = 1L;

    public TagSet() {
    }

    public TagSet(Element owner, Project project, String name) {
        super(owner, project, name);
    }

    private List<Tag> ownedTags = new ArrayList<Tag>(); //! list to preserve ordering
    //EclipseLink: LinkedHashSet Even EAGER below FAILS to prevent substitution with non ordered set.

    @OneToMany(mappedBy = "tagSet",
               cascade = CascadeType.ALL)
    @OrderBy("name ASC") //EclipseLink+Oracle: FAILS to preserve order
    /** 
     * Subsets ownedElements 
     */
    public List<Tag> getOwnedTags() {
        return ownedTags;
    }

    public void setOwnedTags(List<Tag> tags) {
        this.ownedTags = tags;
    }

    /** 
     * IMPORTANT: this will also add an ownedElement !
     */
    public boolean addOwnedTag(Tag tag) {
        if (getOwnedTags().contains(tag)) {
            return false; //set-like behaviour
        }
        addOwnedElement(tag);//!
        return getOwnedTags().add(tag);
    }

    /** 
     * IMPORTANT: this will also remove an ownedElement !
     */
    public boolean removeOwnedTag(Tag tag) {
        removeOwnedElement(tag);//!
        return getOwnedTags().remove(tag);
    }
}
