/* (C) Copyright 2016 Greensoft Pty Ltd, Australia (ACN 148 917 999). All rights reserved. */
package com.greensoft.objectdb.test.entity;

import com.greensoft.objectdb.test.common.All_;
import java.util.ArrayList;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.Transient;

import java.util.List;

/**
 * The universal base class of all entities in the system.
 * 
 * Non-reflective Greensoft ObjectDB test version
 * 
 * @author Darren Kelly (Webel IT Australia for Greensoft Pty Ltd).
 */
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import java.util.LinkedHashSet;
import java.util.Set;
import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.FetchType;
//import javax.persistence.Inheritance;
//import javax.persistence.InheritanceType;

@Entity
//@Inheritance(strategy=InheritanceType.SINGLE_TABLE) //taken as default anyway
public class Element 
//implements Serializable
        extends All_ // For access to common logging methods
{

    private static final long serialVersionUID = 1L;

    private Long id;
 
    @Id
    @GeneratedValue
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    @Override
    /** Warning - this method won't work in the case the id fields are not set */
    public boolean equals(Object object) {
        if (!(object instanceof Element)) {
            return false;
        }
        Element other = (Element) object;
        if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }


    public Element() {
        init();
    }

    public Element(
            Element owner,
            Project project,
            String name) {
        this.owner = owner;
        this.project = project;
        this.name = name;
        init();
    }
 
    private void init() {
        //setDebug(true);
        dateCreated = new java.util.Date();
        dateModified = dateCreated;
    }
    
    private Project project;
    //@ManyToOne(fetch= FetchType.LAZY)//preferred
    @ManyToOne(fetch=FetchType.EAGER) //EXPERIMENTAL
    public Project getProject() {
        return project;
    }
    public void setProject(Project project) {
        this.project = project;
    }
    

    private Element owner;
    @ManyToOne(fetch=FetchType.LAZY) //!
    //@ManyToOne(fetch=FetchType.EAGER)
    public Element getOwner() {
        return owner;
    }
    public void setOwner(Element owner) {
        this.owner = owner;
    }

    private List<Element> ownedElements = new ArrayList<Element>();
    //private List<Element> ownedElements;
    @OneToMany(mappedBy = "owner",
               cascade = CascadeType.ALL
            )
    public List<Element> getOwnedElements() {
        return ownedElements;
    }
    public void setOwnedElements(List<Element> ownedElements) {
        this.ownedElements = ownedElements;
    }

    static final public boolean PERMIT_DUPLICATES = true;
    
    /** Adds the passed Element iff it is non-null.
     *
     * @param element
     * @return
     */
    public boolean addOwnedElement(Element element) {
        if (element == null) {
            log_warn("addOwnedElement: skipping null element.");
            return false;
        }
        //return getOwnedElements().add(element);
        if (PERMIT_DUPLICATES || !getOwnedElements().contains(element)) {
//     //CAUTION: this will not work in the case that the ids are not set, since the equality test
//     //exploits the ids, so it will think all instances without ids are the same.
            return getOwnedElements().add(element);
        } else {
//            //TODO: warn not added
            log_warn("Element.addOwnedElement(" + element + "): EITHER getOwnedElements().contains(element) OR equals comparison failing !");
            return false;
        }
    }
    public boolean removeOwnedElement(Element element) {
        return getOwnedElements().remove(element);
    }


    private String name;
    @Column(length = 1024)
    //EXPERIMENT ONLY @Basic(fetch= FetchType.LAZY)
    public String getName() {
        return name;
    }
    public void setName(String name) {
        this.name = name;
    }

    private String help;
    //@Lob
    //@Basic(fetch = FetchType.LAZY)
    public String getHelp() { 
        return help;
    }
    public void setHelp(String help) {
        this.help = help;
    }
        
    private String body;
    @Lob
    @Basic(fetch = FetchType.LAZY) 
    public String getBody() {
        return body;
    }
    public void setBody(String body) {
        this.body = body;
    }

    private java.util.Date dateCreated;
    @Temporal(javax.persistence.TemporalType.TIMESTAMP) //!
    public java.util.Date getDateCreated() {
        return dateCreated;
    }
    public void setDateCreated(java.util.Date dateCreated) {
        this.dateCreated = dateCreated;
    }

    private java.util.Date dateModified;
    //@Temporal(javax.persistence.TemporalType.DATE)
    @Temporal(javax.persistence.TemporalType.TIMESTAMP) //!
    public java.util.Date getDateModified() {
        return dateModified;
    }

    public void setDateModified(java.util.Date dateModified) {
        //log_debug("setDateModified", "dateModified", dateModified);
        this.dateModified = dateModified;
    }

    private Set<Tag> tags = new LinkedHashSet<Tag>();
    @ManyToMany(mappedBy = "taggedElements")
    public Set<Tag> getTags() {
        return tags;
    }
    public void setTags(Set<Tag> tags) {
        this.tags = tags;
    }
    public boolean addTag(Tag tag) {
        return getTags().add(tag);
    }

    public boolean removeTag(Tag tag) {
        return getTags().remove(tag);
    }


    static final public String $AUTHOR_DEFAULT = "SYSTEM";
    private String author = $AUTHOR_DEFAULT;
    public String getAuthor() {
        return author;
    }
    public void setAuthor(String author) {
        this.author = author;
    }

    private List<Link> links = new ArrayList<Link>();
    @OneToMany(cascade = CascadeType.ALL)
    public List<Link> getLinks() {
        return links;
    }
    public void setLinks(List<Link> links) {
        this.links = links;
    }
    public boolean addLink(Link l) {
        return getLinks().add(l);
    }
    public boolean addLink(String name, String $url) {
        return addLink(new Link(name, $url));
    }
    public boolean addLink(String $url) {
        return addLink(new Link($url));
    }
    public boolean removeLink(Link l) {
        return getLinks().remove(l);
    }

    private Source source;
    @ManyToOne(fetch= FetchType.LAZY)//!
    public Source getSource() {
        return source;
    }
    public void setSource(Source source) {
        this.source = source;
    }

    private List<Constraint> constraints = new ArrayList<Constraint>();
    @ManyToMany(mappedBy = "constrainedElements")
    public List<Constraint> getConstraints() {
        return constraints;
    }
    public void setConstraints(List<Constraint> constraints) {
        this.constraints = constraints;
    }
    public boolean addConstraint(Constraint s) {
        if (PERMIT_DUPLICATES || !getConstraints().contains(s)) {
            return getConstraints().add(s);
        }
        return false;
    }
    public boolean removeConstraint(Constraint s) {
        return getConstraints().remove(s);
    }
    

    private Permissions permissions = new Permissions(); //! here, not in init(): http://tracking.greensoftaustralia.com/node/2331
    @Embedded
    public Permissions getPermissions() {
        return permissions;
    }
    public void setPermissions(Permissions permissions) {
        this.permissions = permissions;
    }
       

    /** Reimplemented to provide the persistence id as object identifier string.
     */
    @Override
    protected String $id() {
        return "" + getId();
    }
    
    @Override
    public String toString() {
        String $project = getProject() == null ? null : getProject().getName();
        return "[class=" + getClass().getSimpleName() + "][id=" + getId() + "][name=" + getName() + "]"//+"[project=" + $project + "]"
                ;
    }

    @Transient
    /** Short output version in format: [id=<id>] "<name>" */
    public String getShortString() {
        //String $project = getProject() == null ? null : getProject().getName();
        return "[id=" + getId() + "] \"" + getName() + "\"";
    }    
    
    private List<Element> unmapped = new ArrayList<Element>();
    @OneToMany(cascade = CascadeType.ALL)
    public List<Element> getUnmapped() {
        return unmapped;
    }
    public void setUnmapped(List<Element> unmapped) {
        this.unmapped = unmapped;
    }
    public boolean addUnmapped(Element e) {
        return unmapped.add(e);
    }
    
}
